/******************************************************************************
 * centralwidget.cpp - A stacked widget to display the various explorers
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2018 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/

#include <QDebug>

#include "mainwindow.h"
#include "regionwidget.h"
#include "mapwidget.h"
#include "mapfile.h"
#include "playerfile.h"
#include "centralwidget.h"

static const bool DEBUG = false;

enum {
	cwNone,
	cwRegion,
	cwMap
};

static int explorerToCw[] = {cwNone, cwNone, cwMap, cwNone, cwRegion, cwRegion, cwRegion};

CentralWidget::CentralWidget(MainWindow* parent)
	: QStackedWidget(),
	  m_mainWindow(parent)
{
	this->addWidget(new QWidget());
	m_regionWidget = new RegionWidget(m_mainWindow, this);
	this->addWidget(m_regionWidget);
	m_mapWidget = new MapWidget(m_mainWindow, this);
	this->addWidget(m_mapWidget);
}

void CentralWidget::explorerTypeChanged(int newExplorerType)
{
	if (DEBUG)
		qDebug() << "CentralWidget:explorerTypeChanged" << newExplorerType << "[" << explorerToCw[newExplorerType] << "]";
	this->setCurrentIndex(explorerToCw[newExplorerType]);
}

void CentralWidget::fileLoaded(NBTFile* file)
{
	MapFile* mf = dynamic_cast<MapFile*>(file);
	if (mf) {
		m_mapWidget->getMapMapWidget()->fileLoaded(mf);
		return;
	}
	PlayerFile* pf = dynamic_cast<PlayerFile*>(file);
	if (pf) {
		m_mapWidget->getMapMapWidget()->fileLoaded(pf);
		return;
	}
}
