/******************************************************************************
 * command.h - Base class for command handling
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2019 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#ifndef COMMAND_H
#define COMMAND_H

#include <QString>
#include <QModelIndex>

const bool DEBUGCOMMAND = true;

class Group;
class NBTFile;
class CommandMan;

class Command
{
public:
	Command();
	virtual	~Command();

	void	setName(const QString& name);
	QString	getName();
	void	setGroup(Group* group);

protected:
	virtual void doCommand() = 0;
	virtual void undoCommand() = 0;
	virtual void redoCommand() = 0;
	/// during destructor, we don't own the data if on the redo list, so null the pointers so they don't double delete
	virtual void disown() = 0;

	QString		m_name;
	Group*		m_group;
	NBTFile*	m_file;
	CommandMan*	m_commandMan;

private:

	friend class CommandMan;
};

inline void Command::setGroup(Group *group) { m_group = group; }
inline void Command::setName(const QString &name) {m_name = name; }
inline QString Command::getName() { return(m_name); }

#endif // COMMAND_H
