/******************************************************************************
 * commanddelete.cpp - User pressed delete in the Resource TreeView
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2019 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#include <QDebug>
#include <QMessageBox>

#include "treemodel.h"
#include "treeitem.h"
#include "commanddelete.h"
#include "commandman.h"
#include "nbtobject.h"

#include "resourcedock.h"

CommandDelete::CommandDelete()
	: m_object(nullptr),
	  m_treeItem(nullptr)
{
}

CommandDelete::~CommandDelete()
{
	if (m_object)
		delete m_object;
	if (m_treeItem)
		delete m_treeItem;
}

void CommandDelete::disown()
{
	m_object = nullptr;
	m_treeItem = nullptr;
}

void CommandDelete::setModelIndex(QModelIndex modelIndex)
{
	m_myrow = modelIndex.row();
	m_parent = modelIndex.parent();
	TreeItem* item = static_cast<TreeItem*>(modelIndex.internalPointer());
	QString name = item->getPath();
	setName(QString("Delete %1").arg(name));
}

void CommandDelete::doCommand()
{
	if (DEBUGCOMMAND)
		qDebug() << "CommandDelete:doCommand";
	TreeModel* tm = m_group->getTreeModel();

	TreeItem* pi = static_cast<TreeItem*>(m_parent.internalPointer());
	//m_parentPath = pi->getPath();
	NBTObject* obj = pi->getObject();
	m_object = obj->removeChild(m_myrow);
	m_treeItem = tm->removeRow(m_parent, m_myrow);
	m_file = m_commandMan->getFile(m_treeItem);
}

void CommandDelete::undoCommand()
{
	if (DEBUGCOMMAND)
		qDebug() << "CommandDelete:undoCommand";
	TreeModel* tm = m_group->getTreeModel();
	TreeItem* pi = static_cast<TreeItem*>(m_parent.internalPointer());
	NBTObject* obj = pi->getObject();
	tm->insertRow(m_parent, m_myrow, m_treeItem);
	obj->insertChild(m_myrow, m_object);
}

void CommandDelete::redoCommand()
{
	if (DEBUGCOMMAND)
		qDebug() << "CommandDelete:redoCommand";
	doCommand();
}
