/******************************************************************************
 * fileman.h - Manage reading files.
 * Note file reading is multithreaded
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2018 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#ifndef FILEMAN_H
#define FILEMAN_H

#include <QThread>
#include "errormsgreceiver.h"

class NBTFile;
class World;
class MainWindow;
class FileReaderThread;

/// Manage reading NBTFiles.
/// Note the multithreaded nature. FileMan runs in the gui thread and
/// the actual file reading is done in numParallelReads FileReaderThread threads.
class FileMan : public QObject, public ErrorMsgReceiver
{
	Q_OBJECT

public:
	FileMan(MainWindow*	parent);

	void	setWorld(World* world);
	void	readFile(NBTFile* file);
	bool	queuedFile(NBTFile* file);

signals:
	void	readFileDone(NBTFile* file);
	void	sendErrorMsg(QString msg);

public slots:
	void	threadReadFileDone(NBTFile* file);
	void	onRailManDone();

protected:
	virtual void errorMsgHandler(const QString& msg);

private:
	NBTFile*	selectFileForReading();
	void		threadReadStart(NBTFile* file);

	MainWindow*	m_mainWindow;
	const int numParallelReads = 4;
	int			m_workerCount;
	QList<NBTFile*>	m_fileQueue;
	int			m_regionsLoaded;
	World*		m_world;
};

class FileReaderThread : public QThread
{
	Q_OBJECT
	friend class FileMan;

signals:
	void	readFileDone(NBTFile* file);

protected:
	virtual void run();

public slots:
	void setupRead(NBTFile* file, FileMan* parent);

private:
	NBTFile*	m_file;
	FileMan*	m_fileMan;
};

#endif // FILEMAN_H
