/******************************************************************************
 * helpman.cpp - I need somebody
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2019 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#include <QDialog>
#include <QVBoxLayout>
#include <QLabel>
#include <QDir>
#include <QMessageBox>
#include <QUrl>
#include <QDesktopServices>

#include "yamceversion.h"
#include "mainwindow.h"
#include "helpman.h"

extern QPixmap* logoPixmap;

HelpMan::HelpMan(MainWindow* parent)
	: m_mainWindow(parent),
	  m_helpDirectory("/usr/share/yamce")
{
}

void HelpMan::onAboutDialog()
{
	QDialog d(m_mainWindow);
	d.setWindowTitle("About yamce");
	QVBoxLayout* mainLayout = new QVBoxLayout();
	QLabel* l;

	QHBoxLayout* hLayout = new QHBoxLayout();
	l = new QLabel();
	l->setPixmap(*logoPixmap);
	l->setScaledContents(true);
	l->setAlignment(Qt::AlignCenter);
	l->setMaximumSize(100,100);
	hLayout->addWidget(l);
	mainLayout->addLayout(hLayout);


	l = new QLabel();
	l->setTextFormat(Qt::RichText);
	l->setText(tr("yamce - Yet Another Minecraft Editor"));
	mainLayout->addWidget(l);
	l = new QLabel(getVersionString());
	mainLayout->addWidget(l);
	l = new QLabel("Copyright(c) 2019 Dick Balaska and BuckoSoft");
	mainLayout->addWidget(l);

	l = new QLabel();
	l->setText("<a href=\"http://www.buckosoft.com/yamce/\">www.buckosoft.com/yamce</a>");
	l->setTextFormat(Qt::RichText);
	l->setTextInteractionFlags(Qt::TextBrowserInteraction);
	l->setOpenExternalLinks(true);
	mainLayout->addWidget(l);

	d.setLayout(mainLayout);
	d.exec();

}

QString HelpMan::getVersionString()
{
	QString s = QString("Version %1").arg(YAMCEVERSION);
	return(s);
}

void HelpMan::onHelp()
{
	QString s = QString("%1/html").arg(m_helpDirectory);
	QDir dir(s);
	QFileInfo fi(dir, "index.html");
	if (!fi.exists()) {
		QMessageBox::critical(m_mainWindow, tr("No help"), tr("Can't open Yamce help.\n File %1 does not exist.").arg(fi.absoluteFilePath()));
		return;
	}
	QUrl url = QUrl::fromLocalFile(fi.absoluteFilePath());
	QDesktopServices::openUrl(url);

}
