/******************************************************************************
 * main.cpp - Startup yamce
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2018 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#include <QApplication>
#include <QDebug>
#include <QTime>

#include "mainwindow.h"
#include "consoleman.h"

extern ConsoleMan* globalConsoleMan;
//static const char*	msgTypeNames[] = {"DEBG", "WARN", "CRIT", "FATL", "INFO"};

void myMessageOutput(QtMsgType type, const QMessageLogContext &context, const QString &msg)
{
	if (globalConsoleMan)
		globalConsoleMan->consoleMessage(type, context, msg);
	else {
		QByteArray localMsg = msg.toLocal8Bit();
		QByteArray time = QTime::currentTime().toString("hh:mm:ss.zzz").toLocal8Bit();
		switch (type) {
		case QtDebugMsg:
#ifdef _DEBUG
			//fprintf(stderr, "[%s] %s (%s:%u)\n", time.constData(), localMsg.constData(), context.file, context.line);
			fprintf(stderr, "[%s] DEBG %s\n", time.constData(), localMsg.constData());
#endif
			break;
		case QtInfoMsg:
			fprintf(stderr, "[%s] INFO %s (%s:%u)\n", time.constData(), localMsg.constData(), context.file, context.line);
			break;
		case QtWarningMsg:
			fprintf(stderr, "[%s] WARN %s (%s:%u)\n", time.constData(), localMsg.constData(), context.file, context.line);
			break;
		case QtCriticalMsg:
			fprintf(stderr, "[%s] CRIT %s (%s:%u)\n", time.constData(), localMsg.constData(), context.file, context.line);
			break;
		case QtFatalMsg:
			fprintf(stderr, "[%s] FATL %s (%s:%u, %s)\n", time.constData(), localMsg.constData(), context.file, context.line, context.function);
			abort();
		}
	}
}

int main(int argc, char *argv[])
{
	qInstallMessageHandler(myMessageOutput);
	QApplication a(argc, argv);
	MainWindow w;
	w.show();

	return a.exec();
}
