/******************************************************************************
 * mapbannerpainter.cpp - Paint the banner on a map
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2018 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#include <QHash>
#include <QColor>
#include <QRect>

#include "mapbannerpainter.h"

static QHash<QString, QColor>	colorMap;

static QPointF rect0tl = QPointF(-3,-5);	// main body
static QPointF rect0br = QPointF( 3, 3);
static QPointF rect1tl = QPointF(-4,-5);	// top line
static QPointF rect1br = QPointF( 4,-5);
static QPointF rect2tl = QPointF(-1, 3);	// nipple
static QPointF rect2br = QPointF( 1, 5);
static QPointF rect3tl = QPointF(-2,-4);	// color swatch
static QPointF rect3br = QPointF( 2, 2);

void MapBannerPainter::init()
{
	colorMap["white"] = QColor(255,252,245);
	colorMap["orange"] = QColor(216,127,51);
	colorMap["magenta"] = QColor(178,76,216);
	colorMap["light_blue"] = QColor(102,153,216);
	colorMap["yellow"] = QColor(229,229,51);
	colorMap["lime"] = QColor(127,204,25);
	colorMap["pink"] = QColor(242,127,165);
	colorMap["gray"] = QColor(76,76,76);
	colorMap["light_gray"] = QColor(153,153,153);
	colorMap["cyan"] = QColor(76,127,153);
	colorMap["purple"] = QColor(127,63,178);
	colorMap["blue"] = QColor(51,76,178);
	colorMap["brown"] = QColor(102,76,51);
	colorMap["green"] = QColor(102,127,51);
	colorMap["red"] = QColor(153,51,51);
	colorMap["black"] = QColor(25,25,25);
}

void MapBannerPainter::draw(QPainter& p, const QPoint &pos, const QString &color, double zoom)
{
	QRectF rect;
	if (zoom < 1.0)
		zoom = 1.0;
	QPointF rect0tl_ = rect0tl*zoom;
	QPointF rect0br_ = rect0br*zoom;
	QPointF rect1tl_ = rect1tl*zoom;
	QPointF rect1br_ = rect1br*zoom;
	QPointF rect2tl_ = rect2tl*zoom;
	QPointF rect2br_ = rect2br*zoom;
	QPointF rect3tl_ = rect3tl*zoom;
	QPointF rect3br_ = rect3br*zoom;
	rect.setTopLeft(pos+rect0tl_);
	rect.setBottomRight(pos+rect0br_);
	p.fillRect(rect, Qt::black);
	rect.setTopLeft(pos+rect1tl_);
	rect.setBottomRight(pos+rect1br_);
	p.fillRect(rect, Qt::black);
	rect.setTopLeft(pos+rect1tl_);
	rect.setBottomRight(pos+rect1br_);
	p.fillRect(rect, Qt::black);
	rect.setTopLeft(pos+rect2tl_);
	rect.setBottomRight(pos+rect2br_);
	p.fillRect(rect, Qt::black);
	rect.setTopLeft(pos+rect3tl_);
	rect.setBottomRight(pos+rect3br_);
	QColor c = colorMap.value(color);
	p.fillRect(rect, c);
}
