/******************************************************************************
 * mapcolordialog.cpp - Choose one of the colors available to the map
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2019 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#include <math.h>
#include <QDebug>
#include <QVBoxLayout>
#include <QPainter>
#include <QMouseEvent>

#include "mainwindow.h"
#include "mapfile.h"
#include "mapcolordialog.h"

static const int numCols = 16;
static const int numRows = 13;

MapColorDialog::MapColorDialog(MainWindow* parent)
	: QDialog(parent),
	  m_mainWindow(parent)
{
	setWindowTitle("yamce - Rail Map Color");
	QVBoxLayout* layout = new QVBoxLayout();
	m_colorWidget = new ColorWidget();
	layout->addWidget(m_colorWidget);
	connect(m_colorWidget, SIGNAL(mapColorSelected(int)), this, SLOT(onMapColorSelected(int)));
	this->setLayout(layout);
}

void MapColorDialog::onMapColorSelected(int mapColor)
{
	emit(mapColorSelected(mapColor));
}

ColorWidget::ColorWidget(QWidget* parent)
	: QWidget(parent)
{

}

QSize ColorWidget::sizeHint() const
{
	QSize size = QSize(numCols*20, numRows*20);
	return(size);
}

void ColorWidget::paintEvent(QPaintEvent*)
{
	const QList<QColor>& colorMap = getMapColorMap();
	QPainter p(this);
	QSize size = this->size();
	QRect rect;
	for (int h=0; h<numRows; h++) {
		for (int w=0; w<numCols; w++) {
			int i=h*numCols+w;
			if (i >= colorMap.size())
				break;
			QColor color = colorMap.at(i);
			double ws = floor((double)(w+0.0)/numCols*size.width());
			double we = ceil((double)(w+1.0)/numCols*size.width());
			double hs = floor((double)(h+0.0)/numRows*size.height());
			double he = ceil((double)(h+1.0)/numRows*size.height());
			rect.setWidth(we-ws);
			rect.setHeight(he-hs);
			rect.moveLeft(ws);
			rect.moveTop(hs);
			p.fillRect(rect, color);
		}
	}
}

void ColorWidget::mousePressEvent(QMouseEvent* event)
{
	qDebug() << "click:" << event;
	QSize size = this->size();
	int yh = size.height()/numRows;
	int r = event->y() / yh;
	int xw = size.width()/numCols;
	int c = event->x() / xw;
	int i = r*numCols+c;

	emit(mapColorSelected(i));

}
