/******************************************************************************
 * mapitem.cpp - Extend NBTFile with map specifics
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2018 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#include <QDebug>
#include <QPixmap>
#include <QPainter>
#include "nbtobject.h"
#include "mapfile.h"
//

static QList<QColor>	colormap {
	QColor(), QColor(), QColor(), QColor(),															// 0-3
	QColor(89, 125, 39), QColor(109, 153, 48), QColor(127, 178, 56), QColor(67, 94, 29),			// 4-7
	QColor(174, 164, 115), QColor(213, 201, 140 ), QColor(247, 233, 163 ), QColor(130, 123, 86),	// 8-11
	QColor(140, 140, 140), QColor(171, 171, 171), QColor(199, 199, 199), QColor(105, 105, 105),		// 12-15
	QColor(180, 0, 0), QColor(220, 0, 0), QColor(255, 0, 0), QColor(135, 0, 0),						// 16-19
	QColor(112, 112, 180), QColor(138, 138, 220), QColor(160, 160, 255), QColor(84, 84, 135),		// 20-23
	QColor(117, 117, 117), QColor(144, 144, 144), QColor(167, 167, 167), QColor(88, 88, 88),		// 24-27
	QColor(0, 87, 0), QColor(0, 106, 0), QColor(0, 124, 0), QColor(0, 65, 0),						// 28-31
	QColor(180, 180, 180), QColor(220, 220, 220), QColor(255, 255, 255), QColor(135, 135, 135),		// 32-35
	QColor(115, 118, 129), QColor(141, 144, 158), QColor(164, 168, 184), QColor(86, 88, 97),		// 36-39
	QColor(106, 76, 54), QColor(130, 94, 66), QColor(151, 109, 77), QColor(79, 57, 40),
	QColor(79, 79, 79), QColor(96, 96, 96), QColor(112, 112, 112), QColor(59, 59, 59),
	QColor(45, 45, 180), QColor(55, 55, 220), QColor(64, 64, 255), QColor(33, 33, 135),				// 48-51
	QColor(100, 84, 50), QColor(123, 102, 62), QColor(143, 119, 72), QColor(75, 63, 38),
	QColor(180, 177, 172), QColor(220, 217, 211), QColor(255, 252, 245), QColor(135, 133, 129),
	QColor(152, 89, 36), QColor(186, 109, 44), QColor(216, 127, 51), QColor(114, 67, 27),			// 60-63
	QColor(125, 53, 152), QColor(153, 65, 186), QColor(178, 76, 216), QColor(94, 40, 114),
	QColor(72, 108, 152), QColor(88, 132, 186), QColor(102, 153, 216), QColor(54, 81, 114),
	QColor(161, 161, 36), QColor(197, 197, 44), QColor(229, 229, 51), QColor(121, 121, 27),
	QColor(89, 144, 17), QColor(109, 176, 21), QColor(127, 204, 25), QColor(67, 108, 13),			// 76-79
	QColor(170, 89, 116), QColor(208, 109, 142), QColor(242, 127, 165), QColor(128, 67, 87),
	QColor(53, 53, 53), QColor(65, 65, 65), QColor(76, 76, 76), QColor(40, 40, 40),
	QColor(108, 108, 108), QColor(132, 132, 132), QColor(153, 153, 153), QColor(81, 81, 81),		// 88-91
	QColor(53, 89, 108), QColor(65, 109, 132), QColor(76, 127, 153), QColor(40, 67, 81),
	QColor(89, 44, 125), QColor(109, 54, 153), QColor(127, 63, 178), QColor(67, 33, 94),			// 96-99
	QColor(36, 53, 125), QColor(44, 65, 153), QColor(51, 76, 178), QColor(27, 40, 94),
	QColor(72, 53, 36), QColor(88, 65, 44), QColor(102, 76, 51), QColor(54, 40, 27),
	QColor(72, 89, 36), QColor(88, 109, 44), QColor(102, 127, 51), QColor(54, 67, 27),				// 108-111
	QColor(108, 36, 36), QColor(132, 44, 44), QColor(153, 51, 51), QColor(81, 27, 27),
	QColor(17, 17, 17), QColor(21, 21, 21), QColor(25, 25, 25), QColor(13, 13, 13),
	QColor(176, 168, 54), QColor(215, 205, 66), QColor(250, 238, 77), QColor(132, 126, 40),
	QColor(64, 154, 150), QColor(79, 188, 183), QColor(92, 219, 213), QColor(48, 115, 112),			// 124-127
	QColor(52, 90, 180), QColor(63, 110, 220), QColor(74, 128, 255), QColor(39, 67, 135),
	QColor(0, 153, 40), QColor(0, 187, 50), QColor(0, 217, 58), QColor(0, 114, 30),
	QColor(91, 60, 34), QColor(111, 74, 42), QColor(129, 86, 49), QColor(68, 45, 25),				// 136-139
	QColor(79, 1, 0), QColor(96, 1, 0), QColor(112, 2, 0), QColor(59, 1, 0),
	QColor(147, 124, 113), QColor(180, 152, 138), QColor(209, 177, 161), QColor(110, 93, 85),
	QColor(112, 57, 25), QColor(137, 70, 31), QColor(159, 82, 36), QColor(84, 43, 19),
	QColor(105, 61, 76), QColor(128, 75, 93), QColor(149, 87, 108), QColor(78, 46, 57),				// 152-155
	QColor(79, 76, 97), QColor(96, 93, 119), QColor(112, 108, 138), QColor(59, 57, 73),
	QColor(131, 93, 25), QColor(160, 114, 31), QColor(186, 133, 36), QColor(98, 70, 19),
	QColor(72, 82, 37), QColor(88, 100, 45), QColor(103, 117, 53), QColor(54, 61, 28),				// 164-167
	QColor(112, 54, 55), QColor(138, 66, 67), QColor(160, 77, 78), QColor(84, 40, 41),
	QColor(40, 28, 24), QColor(49, 35, 30), QColor(57, 41, 35), QColor(30, 21, 18),
	QColor(95, 75, 69), QColor(116, 92, 84), QColor(135, 107, 98), QColor(71, 56, 51),				// 176-179
	QColor(61, 64, 64), QColor(75, 79, 79), QColor(87, 92, 92), QColor(46, 48, 48),
	QColor(86, 51, 62), QColor(105, 62, 75), QColor(122, 73, 88), QColor(64, 38, 46),
	QColor(53, 43, 64), QColor(65, 53, 79), QColor(76, 62, 92), QColor(40, 32, 48),
	QColor(53, 35, 24), QColor(65, 43, 30), QColor(76, 50, 35), QColor(40, 26, 18),					// 192-195
	QColor(53, 57, 29), QColor(65, 70, 36), QColor(76, 82, 42), QColor(40, 43, 22),
	QColor(100, 42, 32), QColor(122, 51, 39), QColor(142, 60, 46), QColor(75, 31, 24),
	QColor(26, 15, 11), QColor(31, 18, 13), QColor(37, 22, 16), QColor(19, 11, 8)					// 204-207
	};

const QList<QColor>& getMapColorMap() { return(colormap); }

MapFile::MapFile()
	: m_pixmap(nullptr),
	  m_scale(-1),
	  m_dimension(0),
	  m_explorerMap(false),
	  m_imageProcessed(false),
	  m_railsApplied(false),
	  m_imageDisplayed(true)
{
}

MapFile::~MapFile()
{
	if (m_pixmap)
		delete m_pixmap;
	while (!m_mapBanners.isEmpty())
		delete m_mapBanners.takeLast();
}

bool MapFile::load(ErrorMsgReceiver* receiver)
{
	bool res = super::load(receiver);
	if (!res)
		return(false);
	return(parseMap(receiver));
}

bool MapFile::parseMap(ErrorMsgReceiver*)
{
	if (m_pixmap)
		delete m_pixmap;
	m_pixmap = nullptr;
	NBTObject* sobj = m_object->findChild("*:data:scale");
	if (!sobj) {
		qInfo() << "MapFile:load" << m_fileName << "no *:data:scale";
		return(false);
	}
	m_scale = sobj->d_int;
	sobj = m_object->findChild("*:data:xCenter");
	if (!sobj) {
		qInfo() << "MapFile:load" << m_fileName << "no *:data:xCenter";
		return(false);
	}
	m_pos.setX(sobj->d_int);
	sobj = m_object->findChild("*:data:zCenter");
	if (!sobj) {
		qInfo() << "MapFile:load" << m_fileName << "no *:data:zCenter";
		return(false);
	}
	m_pos.setY(sobj->d_int);

	sobj = m_object->findChild("*:data:dimension");
	if (!sobj) {
		qInfo() << "MapFile:load" << m_fileName << "no *:data:dimension";
		return(false);
	}
	m_dimension = sobj->d_int;

	NBTObject*	data = m_object->findChild("*:data:colors");
	if (!data) {
		qCritical() << "MapItem:load failed to find data:colors";
		return(false);
	}
	m_pixmap = new QPixmap(128, 128);
	QPainter p(m_pixmap);
	for (int w=0; w<128; w++) {
		for (int h=0; h<128; h++) {
			quint8 pixel = data->d_byte_array->at(h*128+w);
			p.setPen(colormap[pixel]);
			p.drawPoint(w, h);
		}
	}
	while (!m_mapBanners.isEmpty())
		delete m_mapBanners.takeLast();
	data = m_object->findChild("*:data:banners");
	if (data) {
		NList* banners = data->d_list;
		if (banners->objcount) {
			QList<NBTObject*>* blist = banners->d_compound;
			foreach (NBTObject* obj, *blist) {
				MapBanner* mb = new MapBanner();
				NBTObject* o = obj->findChild("*:Pos");
				if (o) {
					NBTObject* xz = o->findChild("*:X");
					if (xz)
						mb->m_pos.setX(xz->d_int);
					xz = o->findChild("*:Z");
					if (xz)
						mb->m_pos.setY(xz->d_int);
					xz = obj->findChild("*:Color");
					if (xz)
						mb->m_color = *xz->d_string;
					m_mapBanners.append(mb);
				} else {
					delete mb;
				}
			}
		}
	}
	data = m_object->findChild("*:data:unlimitedTracking");
	if (data)
		m_explorerMap = data->d_byte;
	return(true);
}
