/******************************************************************************
 * nbtobject.h - A single NBT object
 * See: https://minecraft.gamepedia.com/NBT_format
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2018 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#ifndef NBTOBJECT_H
#define NBTOBJECT_H

#include <QObject>

#include "nlist.h"

class NBTObject;

extern QStringList	nbtTypeNames;

enum NBT_Tag {
	TAG_End,			// 0
	TAG_Byte,			// 1
	TAG_Short,			// 2
	TAG_Int,			// 3
	TAG_Long,			// 4
	TAG_Float,			// 5
	TAG_Double,			// 6
	TAG_Byte_Array,		// 7
	TAG_String,			// 8
	TAG_List,			// 9
	TAG_Compound,		// 10
	TAG_Int_Array,		// 11
	TAG_Long_Array		// 12
};


class NBTObject {

public:
	NBTObject() { d_compound = nullptr; size = 0;}
	~NBTObject();

	NBTObject& operator=(const NBTObject& other);

	NBTObject*		findChild(const QString& name);
	NBTObject*		removeChild(int row);
	void			insertChild(int row, NBTObject* child);

//	QString			getPath();
	int				getChildIndex(void* child);

	void			setValue(QVariant value);
	QVariant		getValue();

	QString name;

	/// The NBT_Tag of this object.
	qint8	type;

	int	size;		// for the array types
	union {
		qint8		d_byte;
		qint16		d_short;
		qint32		d_int;
		qint64		d_long;
		float		d_float;
		double		d_double;
		QByteArray*	d_byte_array;
		QString*	d_string;
		NList*		d_list;
		QList<NBTObject*>*	d_compound;
		qint32*		d_int_array;
		qint64*		d_long_array;
	};

public:
	static void init();		// boot initialization

};

#endif // NBTOBJECT_H
