/******************************************************************************
 * nlist.cpp - A sized list of data
 * See: NBTObject.h and https://minecraft.gamepedia.com/NBT_format
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2019 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#include <QDebug>
#include "nlist.h"
#include "nbtobject.h"

NList::~NList() {
	switch (objtype) {
	case TAG_Compound:
		if (d_compound) {
			while (!d_compound->isEmpty())
				delete d_compound->takeFirst();
			delete d_compound;
		}
		break;
	case TAG_List:
		if (d_list) {
			while (!d_list->isEmpty())
				delete d_list->takeLast();
			delete d_list;
		}
		break;
	case TAG_Int_Array:
		if (d_int_array) {
			for (int i=0; i<objcount; i++)
				delete[] d_int_array[i];
			delete[] d_int_array;
		}
		break;
	case TAG_Long_Array:
		if (d_long_array) {
			for (int i=0; i<objcount; i++)
				delete[] d_long_array[i];
			delete[] d_long_array;
		}
		break;
	case TAG_Byte:
		delete[] d_byte;
		break;
	case TAG_Short:
		delete[] d_short;
		break;
	case TAG_Int:
		delete[] d_int;
		break;
	case TAG_String:
		delete d_string;
		break;
	case TAG_Double:
		delete[] d_double;
		break;
	case TAG_Float:
		delete[] d_float;
		break;
	case TAG_End:			// emptyish list?
		break;
	default:
		//delete[] data;
		qCritical() << "~NList unhandled type" << objtype;
		break;
	}
}

NBTObject* NList::removeRow(int row)
{
	NBTObject* obj = nullptr;
	switch(objtype) {
	case TAG_Compound:
		obj = d_compound->at(row);
		d_compound->removeAt(row);
		objcount--;
		break;
	default:
		qCritical() << "NList:removeRow unhandled list type" << objtype;
	}
	return(obj);
}

void NList::insertRow(int row, NBTObject* child)
{
	switch(objtype) {
	case TAG_Compound:
		d_compound->insert(row, child);
		objcount++;
		return;
	default:
		qCritical() << "NList:insertRow unhandled list type" << objtype;

	}
}
