/******************************************************************************
 * nlist.h - A sized list of data
 * See: NBTObject.h and https://minecraft.gamepedia.com/NBT_format
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2019 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#ifndef NLIST_H
#define NLIST_H

#include <QObject>

class NBTObject;

class NList {
public:
	NList() { data = nullptr; mallocCount = 0;}
	~NList();

	NBTObject*	removeRow(int row);
	void		insertRow(int row, NBTObject* child);

	qint8	objtype;
	int		objcount;
	int		mallocCount;
	union {
		quint8*				data;
		qint8*				d_byte;
		qint16*				d_short;
		qint32*				d_int;
		qint64*				d_long;
		QList<NBTObject*>*	d_compound;
		QStringList*		d_string;
		QList<NList*>*		d_list;
		double*				d_double;
		float*				d_float;
		qint32**			d_int_array;
		qint64**			d_long_array;
	};

};

#endif // NLIST_H
