/******************************************************************************
 * regionmapwidget.h - The central widget that displays the world map
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2018 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#ifndef REGIONMAPWIDGET_H
#define REGIONMAPWIDGET_H

#include <QLabel>
#include <QScrollArea>

#include "world.h"

class MainWindow;
class World;


class RegionMapWidget : public QWidget
{
	Q_OBJECT

public:
	explicit RegionMapWidget(MainWindow* mainWindow, QScrollArea* parent);
	void	setWorld(World* newWorld);
	void	redraw() { update(); }

signals:
	void	selectRegion(int x, int z);

protected:
	  void paintEvent(QPaintEvent* event) override;
	  void resizeEvent(QResizeEvent* event) override;
	  void mousePressEvent(QMouseEvent* event) override;
	  //bool event(QEvent *event) override;


private:
	  void		paintOverworldRegions();

	int				m_explorerType;
	World*			m_world;
//	QScrollArea*	m_parent;
//	MainWindow*		m_mainWindow;
	int				m_regionSizeX;		// number of horizontal regions
	int				m_regionSizeY;		// number of vertical regions
	double			m_zoom;

	friend class RegionMapScrollArea;
};


class RegionMapScrollArea : public QScrollArea
{
	Q_OBJECT

public:
	RegionMapScrollArea(MainWindow* mainWindow, QWidget* parent = nullptr);

	RegionMapWidget*	m_widget;

public slots:
	  void	zoomValueChanged(double value);
	  void	explorerTypeChanged(int newExplorerType);

protected:
	void resizeEvent(QResizeEvent *event) override;
};

/// @brief The main widget for displaying the RegionMaps
class RegionWidget : public QWidget
{
public:
	explicit RegionWidget(MainWindow* mainWindow, QWidget* parent = nullptr);

private:
	RegionMapScrollArea*	m_regionMapScrollArea;

	friend class MainWindow;
};

#endif // REGIONMAPWIDGET_H
