/******************************************************************************
 * statusbar.cpp - The bottom statusbar
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2018 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/

#include "progressset.h"
#include "statusbar.h"

StatusBar::StatusBar(MainWindow* mainWindow)
	: m_mainWindow(mainWindow)
{
	addWidget(&m_statusLabel);
	addPermanentWidget(&m_progressBar);
	m_progressBar.setVisible(false);
}

void StatusBar::setProgressText(const QString& text, int total)
{
	m_progressText = text;
	m_progressToDo = total;
	m_progressDone = -1;
	m_progressBar.setRange(0, m_progressToDo);
	m_progressBar.setVisible(true);
	incProgressDone();
}

void StatusBar::incProgressDone()
{
	m_progressDone++;
	QString s = QString(m_progressText).arg(m_progressDone).arg(m_progressToDo);
	m_progressBar.setValue(m_progressDone);
	m_progressBar.setFormat(s);
	if (m_progressDone >= m_progressToDo)
		m_progressBar.setVisible(false);
}

void StatusBar::setProgress(ProgressSet* set)
{
	if (set->m_done >= set->m_total)
		m_progressBar.setVisible(false);
	else {
		m_progressBar.setVisible(true);
		m_progressBar.setRange(0, set->m_total);
		m_progressBar.setValue(set->m_done);
		QString s = QString(set->m_text).arg(set->m_done).arg(set->m_total);
		m_progressBar.setFormat(s);
	}
}

void StatusBar::showMessage(const QString &text)
{
	m_statusLabel.setText(text);
}
