/******************************************************************************
 * syncman.h - Keep yamce synchronized with a running Minecraft world
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2019 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#ifndef SYNCMAN_H
#define SYNCMAN_H

#include <QObject>
#include <QDateTime>
#include <QFileInfoList>
#include <QTimer>

class QFileSystemWatcher;

class DelayedRead;
class MainWindow;
class NBTFile;
class World;

class SyncMan : public QObject
{
	Q_OBJECT

public:
	explicit SyncMan(MainWindow* mainWindow);
	~SyncMan();

	bool	isActive();
public slots:
	void	onSynchronize(bool checked);

private slots:
	void	onFileChanged(const QString& path);
	void	onDirectoryChanged(const QString& path);
	void	onTimeout();

private:
	void	setWorld(World* world);
	void	seedMapList();
	void	addMapsToWatch();
	void	updatePlayerPositions();
	void	mapDirectoryChanged();
	void	mapChanged(const QFileInfo& fileInfo);
	void	queueDelayedRead(NBTFile* file);

	MainWindow*			m_mainWindow;
	World*				m_world;
	QFileSystemWatcher*	m_fileSystemWatcher;
	QDateTime			m_lastSync;
	//QStringList			m_mapList;
	QString				m_worldDir;
	QString				m_mapDir;
	QString				m_regionDir;
	QString				m_playerDir;
	QFileInfoList		m_mapsFileInfoList;
	QFileInfoList		m_playersFileInfoList;
	QTimer				m_timer;
	QList<DelayedRead*>	m_readQueue;
};

inline bool SyncMan::isActive() { return(m_fileSystemWatcher != nullptr); }


/// We get notified that a file has changed before Minecraft is finished writing
/// the file.  So, queue the read and delay reading for 500ms.
class DelayedRead {
public:
	DelayedRead() { m_tick = 5; }
	int			m_tick;
	NBTFile*	m_file;
};


#endif // SYNCMAN_H
