/******************************************************************************
 * xyz.cpp - a 3D point
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2019 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#include <QDebug>
#include <QStringList>

#include "xyz.h"

QString XYZ::toString(bool useSpaces)
{
	QString s;
	if (useSpaces)
		s = QString("%1 %2 %3").arg(x).arg(y).arg(z);
	else
		s = QString("%1,%2,%3").arg(x).arg(y).arg(z);
	return(s);
}

XYZ& XYZ::fromString(const QString& in, bool* usesSpaces, bool* success)
{
	QStringList tokens = in.split(" ");
	if (tokens.size() == 3) {
		if (usesSpaces)
			*usesSpaces = true;
	} else {
		tokens = in.split(",");
		if (tokens.size() == 3 && usesSpaces)
			*usesSpaces = false;
	}
	if (tokens.size() != 3) {
		return(*this);
	}
	bool ok;
	//int x,y,z;
	x = tokens[0].toInt(&ok);
	if (ok)
		y = tokens[1].toInt(&ok);
	if (ok)
		z = tokens[2].toInt(&ok);
	if (success)
		*success = ok;
	return(*this);
}

QDebug &operator<<(QDebug & stream, const XYZ& xyz)
{
	stream << "XYZ:" << xyz.x << "/" << xyz.y << "/" << xyz.z;
	return(stream);
}

QDataStream& operator<<(QDataStream& stream, const XYZ& xyz)
{
	stream << QVariant(xyz.x) << QVariant(xyz.y) << QVariant(xyz.z);
	return(stream);
}

QDataStream& operator>>(QDataStream& stream, XYZ& xyz)
{
	QVariant x,y,z;
	stream >> x >> y >> z;
	xyz.x = x.toInt();
	xyz.y = y.toInt();
	xyz.z = z.toInt();
	return(stream);
}
